include <threads.scad>;

//true if rendering the threads.
fullRendering = false;
fn = 120;

funnelRadius = 80;
funnelHeight = 125;


module triangleR(a, b, h, c){
  linear_extrude(height = h, center = c, convexity = 10)
  resize([a, b])
  polygon([[0,0], [1,0], [0,1]], paths=[[0,1,2]]);
}

module triangleRa(a, angle, h, c){
  rotate([90, 0, 0])
  linear_extrude(height = h, center = c, convexity = 10)
  resize([a, a*tan(angle)])
  polygon([[0,0], [1,0], [0,1]], paths=[[0,1,2]]);
}

module triangleI(x, y, z, c){
  rotate([90, 0, 0])
  linear_extrude(height = y, center = c, convexity = 10)
  resize([x, z])
  polygon([[-1,0], [1,0], [0,1]], paths=[[0,1,2]]);
}

module bucket(){
  difference(){
    union(){
      difference(){
        triangleI(70, 20, 25, true);
        
        translate([-70/4-1/2, 0, (25+1)/2]) cube([70/2, 20-2, 25-1], center = true);
        translate([70/4+1/2, 0, (25+1)/2]) cube([70/2, 20-2, 25-1], center = true);
      }
      
      triangleI(25, 20, 15, true);
    }
    
    translate([0, 0, 3]) rotate([90, 0, 0]) cylinder(d=3.25, h=20, center=true, $fn=30);
  }
  
  //magnet
  translate([0, 10+2/2, 21])
  rotate([90, 0, 0])
  difference(){
    cylinder(d=6.25, h=2, center=true, $fn=30);
    cylinder(d=4.25, h=2, center=true, $fn=30);
  }  
}

module funnel(){
  translate([0, 0, 10])
  difference(){
    union(){
      translate([0, 0, funnelHeight]) mirror([0, 0, 1]) rotate_extrude(angle = 360, convexity = 2, $fn=fn) difference(){
        resize([funnelRadius+2, funnelRadius+2])  polygon([[0,0], [1,0], [0,1]], paths=[[0,1,2]]);
        resize([funnelRadius, funnelRadius])  polygon([[0,0], [1,0], [0,1]], paths=[[0,1,2]]);
      }
      
      difference(){
        cylinder(r=funnelRadius+2, h=funnelHeight, center=false, $fn=fn);
        cylinder(r=funnelRadius, h=funnelHeight, center=false, $fn=fn);
      } 

      translate([0, 0, funnelHeight-funnelRadius-2-10/2+7/2]) cylinder(d=7, h=10, center=true, $fn=30);
    }
    translate([0, 0, funnelHeight-funnelRadius-2-10/2+7/2]) cylinder(d=5, h=14, center=true, $fn=30);
  }
  
  
  difference(){
    cylinder(r=funnelRadius+2, h=25, center=false, $fn=fn);
    translate([0, 0, 11]) cylinder(r1=funnelRadius-9, r2=funnelRadius, h=14, center=false, $fn=fn);
    metric_thread (diameter=2*(funnelRadius-3)+1, pitch=6, angle=45, length=11, leadin=2, internal = true, test = !fullRendering);   
  } 

    
}


module base(){
  translate([0, 0, -10]) 
  difference(){
    cylinder(r=funnelRadius+2, h=10, center=false, $fn=fn);    
    
    //water holes
    for(dy = [-2:1:2], dx = [-1.5, -1, 0, 1, 1.5]){
      translate([-39*dx, 5*dy, 10/2]) cube([14, 4, 10], center = true);
    }
    for(dy = [-1:1:1], dyy = [-1,1]){
      translate([0, 6*dy+dyy*40, 10/2]) cube([40, 4, 10], center = true);
    }    
    
    //foot screw holes
    for(rz = [0:90:270]) rotate([0, 0, 45+rz]) translate([funnelRadius-18, 0, 0]) cylinder(d=4.25, h=10, center=false, $fn=30);
    
    //cables hole
    translate([0, -(funnelRadius-18), 0])  cylinder(d=12.5, h=10, $fn=30, center=false);
    
    //extension arm screws
    for(rz=[-25, 25]) rotate([0, 0, rz]) translate([0, -(funnelRadius-18), 0]) cylinder(d=4.5, h=10, $fn=30, center=false);
      
    //adjust bucket screw
    for(dx=[-1,1]) translate([dx*25, 0, 5]) cylinder(d=2.9, h=5, center=false, $fn=15);
    
  }
  
  //extension arm screws
  for(rz=[-25, 25]) rotate([0, 0, rz]) translate([0, -(funnelRadius-18), 0]) difference(){
    cylinder(d=12, h=3, center=false, $fn=30);
    cylinder(d=8, h=3, center=false, $fn=30);
  }
  
  //foot screw holes
  for(rz = [0:90:270]) rotate([0, 0, 45+rz]) translate([funnelRadius-18, 0, 0]) difference(){
    cylinder(d=12, h=5, center=false, $fn=30);
    cylinder(d=4.25, h=5, center=false, $fn=30);
    translate([0, 0, 5-3]) cylinder(d=8, h=3, center=false, $fn=30);    
  }
  
  
  //sensor screw holes
  difference(){
    hull(){
       translate([-15, 22, 0]) cylinder(d=10, h=5, center=false, $fn=30);
       translate([15, 22, 0]) cylinder(d=10, h=5, center=false, $fn=30);
    }
    for(dx = [-1,1]){
      translate([15*dx, 22, 0]) cylinder(d=2.9, h=5, center=false, $fn=30);
    }
  }
  
    
  //bucket holder    
  translate([0, 0, 16/2]) difference(){
      union(){
          cube([92+2*2, 24+2*2, 16], center = true);
          cube([24, 24+2*5, 16], center = true);
      }
      cube([92, 24, 16], center = true);
      
      translate([0, 0, (16-6)/2]) cube([24, 24+2*5, 6], center = true);
      
      //holes for axis
      for(dx=[-1,1], dy=[-1,1]) translate([dx*18/2, dy*(34-5)/2, -3]) cylinder(d=2.45, h=5, center=false, $fn=15);
  }
    
    
  //default bucket stop
  difference(){
    for(dx = [-1, 1]) {
      translate([22*dx, 0, 3]) scale([dx, 1, 1]) {
        translate([10/2, 0, -3/2]) cube([10, 24, 3], center = true);
        triangleRa(10, 20, 24, true);
      }
    }
    translate([0, 0, 10/2]) cube([70, 20-2*2, 10], center = true);
  }
  
  
  //sma pad
  translate([-58, 20, 0]) rotate([0, 0, 65]) difference(){
    hull(){
        translate([0, 0, 0]) cylinder(d=12, h=6, $fn=30, center=false);
        translate([12, 0, 0]) cylinder(d=12, h=6, $fn=30, center=false);
    }
    translate([12, 0, 0]) cylinder(d=2.9, h=6, $fn=15, center=false);
  }
  
  //pcb box holder
  translate([0, -21, 15/2]) difference(){
    cube([40, 12, 15], center=true);
    translate([0, (12-5)/2, 0]) cube([28, 5, 15], center=true);
    translate([0, 0, (15-12)/2]) cube([34.5, 6.33, 12], center=true);
    translate([0, 0, 1.75]) rotate([90, 0, 0]) cylinder(d=4.25, h=20, $fn=30, center=true);
  }
  
  
  
  //thread
  difference(){
    metric_thread (diameter=2*(funnelRadius-3)-1, pitch=6, angle=45, length=10, internal=false, test=!fullRendering);
    cylinder(d=2*(funnelRadius-9), h=12, center=false, $fn=fn);    
  } 

}

module bucketHolders(){
    difference(){
        translate([0, 0, 10+9/2]) cube([24, 24+2*5, 9], center = true);
        translate([0, 0, 10+9/2]) cube([24, 24, 9], center = true);
      
        for(dx=[-1,1]) translate([dx*18/2, 0, 10+9]) cube([6, 24+2*5, 2], center = true);

        
        translate([0, 0, 10+8]) rotate([90, 0, 0]) cylinder(d=3.5, h=20+2*2+2*3+0.5, center=true, $fn=30);
        
        for(dx=[-1,1], dy=[-1,1]) translate([dx*18/2, dy*(34-5)/2, 10]) cylinder(d=2.75, h=8, center=false, $fn=15); //M2.5x12
        //for(dx=[-1,1], dy=[-1,1]) translate([dx*18/2, dy*(34-5)/2, 17]) cylinder(d=5, h=2.5, center=false, $fn=15); //M2.5x12
    }
    
}

module extensionHarm(length){
    w = length+3;
    solarCellAxe = (funnelRadius-18)+length;
    difference(){
        minkowski(){
            union(){
                rotate([0, 0, 180-50/2]) union(){
                    rotate_extrude(angle=50, $fn=fn) translate([funnelRadius-18, 0, 0]) square([18-2*2, 10-2], center=true);
                    for(rz=[0, 50]) rotate([0, 0, rz]) translate([funnelRadius-18, 0, 0]) 
                        cylinder(d=18-2*2, h=10-2, $fn=30, center=true);        
                }
                translate([-w/2-(funnelRadius-18), 0, 0]) cube([w, 25, 10-2], center=true);
            }
            sphere(r=2, $fn=30);
        }
        translate([0, 0, (10-2)/2]) cylinder(d=2*funnelRadius+2*2+0.25, h=2, $fn=fn);
        
        //solar cell holder
        translate([-solarCellAxe-1, 0, -1]) cube([10, 20.5, 10], center=true);
        translate([-solarCellAxe, 0, 0]) rotate([90, 0, 0]) cylinder(d=3.5, h=30, center=true, $fn=20);
        for(dy=[-1,1]) translate([-solarCellAxe, dy*29/2, 0]) rotate([90, 0, 0]) cylinder(d=6, h=4, center=true, $fn=20);
        
        //cable hole
        translate([-(funnelRadius-18), 0, 2]) cylinder(d=9, h=8, $fn=30, center=true);
        
        //mounting screws M4
        for(rz=[-25, 25]) rotate([0, 0, rz]) translate([-(funnelRadius-18), 0, 0]) {
            translate([0, 0, -4]) cylinder(d=3.5, h=8, $fn=30, center=false);
        }         

        //cable path
        hull(){
            translate([-solarCellAxe, 0, 0]) cylinder(d=6, h=4, $fn=30, center=true);
            translate([-(funnelRadius-18), 0, 0]) cylinder(d=6, h=4, $fn=30, center=true);
        }      
        translate([-solarCellAxe+4, 0, 0]) rotate([90, 0, 0]) cylinder(d=8, h=10, $fn=30, center=true);
        
    }
     
   //top cable path 
    translate([-(funnelRadius-18), 0, 10/2+4])
    difference() {
        cylinder(d=12, h=10, $fn=30, center=true);
        cylinder(d=9, h=10, $fn=30, center=true);
    }    
}

module solarCellHolder(){   
    translate([-25, 0, 0])
    difference(){
        union(){
            minkowski(){
                cube([31, 54, 6], center=true);
                sphere(r=2, $fn=30);
            }
            
            translate([20, 0, -1]) rotate([90, 0, 0])
            hull(){
                translate([-5, 0, 0]) cylinder(d=8, h=20, center=true, $fn=30);
                translate([5, 0, 0]) cylinder(d=8, h=20, center=true, $fn=30);
            }
            
        }    
        //cell
        translate([0, 0, (6+2*2-3)/2]) cube([31, 54, 3], center=true);
        
        //inside
        translate([0, 0, (6+2*2-8)/2]) cube([30-2*5, 53-2*5, 8], center=true);
        translate([0, 0, (6+2*2-6)/2]) cube([6, 52.5, 4], center=true);
        
        //cable
        translate([15, 0, -1]) rotate([0, 90, 0]) cylinder(d=4, h=30, center=true, $fn=30);
        translate([27, 0, -1]) cube([4, 5, 10], center=true);        
        
        //screws
        for(dy=[-1,1]) translate([25, dy*7, -1]) rotate([90, 0, 0]) cylinder(d=3, h=6, center=true, $fn=30);
    }    
}


module sensor(){
  difference(){
    hull(){
       translate([-15, 0, 0]) cylinder(d=10, h=5, center=false, $fn=30);
       translate([15, 0, 0]) cylinder(d=10, h=5, center=false, $fn=30);
    }
    for(dx = [-1,1]){
      translate([15*dx, 0, 0]) cylinder(d=3, h=5, center=false, $fn=30);
      translate([15*dx, 0, 3]) cylinder(d=6, h=2, center=false, $fn=30);
    }
  }
  
  translate([0, -3, 22.5])
  difference(){
    union(){
      translate([0, 3, 3]) cube([20, 10, 45], center = true);
      
      translate([0, -3, 12])
      difference(){
          hull(){
            translate([0, 3/2, 0]) cube([5, 0.1, 14+6], center = true);
            translate([0, -3/2+0.1/2, 0]) cube([5, 0.1, 14], center = true);
          }
          translate([0, -3/2+2.25/2, 0]) cylinder(d=2.25, h=8, center=true, $fn=30);
          translate([0, -3/2+2.25/4, 0]) cube([2.25, 2.25/2, 8], center = true);
          translate([0, -3/2+2.25/4, 0]) cube([0.75, 2.25/2, 12], center = true);
      }
      
    }
          
    translate([0, 0, 12+6-0.5]) cube([1, 16, 1], center = true);
    translate([0, 0, 12-6+0.5]) cube([1, 16, 1], center = true);

    translate([0, 5.5, 8]) cube([3, 5, 30], center = true);

  }
  
}

module pcbHolder(){
  difference(){
    union(){
             
      translate([0, 0, 25/2]) minkowski(){
        cube([42, 42, 25], center=true);
        cylinder(r=2.5+1.4+2, h=1, center=false, $fn=30);
      } 
      
      translate([0, 0, 37/2]) minkowski(){
        cube([42, 42, 37], center=true);
        cylinder(r=2.5+1.4, h=1, center=false, $fn=30);
      } 
               
      //cover pads
      for(rz = [0:90:270]) rotate([0, 0, 45+rz]) hull(){
        translate([0, 0, 0]) cylinder(d=8, h=26, center=false, $fn=30);
        translate([37, 0, 0]) cylinder(d=8, h=26, center=false, $fn=30);
      }
      

    }
    
    //inside
    translate([0, 0, 35/2+2])  minkowski(){
      cube([42, 42, 35], center=true);
      cylinder(r=2.5, h=1, center=false, $fn=30);
    } 
    
 
    
    //cover holes
    for(rz = [0:90:270]) rotate([0, 0, 45+rz]) hull(){
      translate([37, 0, 16]) cylinder(d=2.9, h=10, center=false, $fn=30);
    }
         
    
    //sensor cable hole 
    for(dx=[-1,1]) translate([dx*9, -24, 21]) rotate([90, 0, 0]) minkowski(){ 
      cube([11, 5, 4], center=true);
      cylinder(r=1, h=1, center=false, $fn=15);
    }
  }  
  
  //pcb holders
  for(dx=[-1,1], dy=[-1,1]) translate([dx*38/2, dy*38/2, 2])
  difference(){
    cylinder(d=6, h=4, center=false, $fn=15);
    cylinder(d=2.5, h=4, center=false, $fn=15);
  }    
  
  //mounting pad
  translate([32, 0, 6/2])
  difference(){
    minkowski(){
      cube([10, 30, 5], center=true);
      cylinder(r=2, h=1, $fn=30, center=true);
    }
    translate([3/4, 0, 0]) cylinder(d=3.75, h=6, $fn=30, center=true);
  }
}

module pcbCover(){
  difference(){    
    union(){
      translate([0, 0, 14/2]) minkowski(){
        cube([42, 42, 14], center=true);
        cylinder(r=2.5+1.4+2, h=1, center=false, $fn=30);
      } 
      for(rz = [0:90:270]) rotate([0, 0, 45+rz]) hull(){
        translate([0, 0, 0]) cylinder(d=8, h=15, center=false, $fn=30);
        translate([37, 0, 0]) cylinder(d=8, h=15, center=false, $fn=30);
      }
    }
    
    translate([0, 0, 11/2]) minkowski(){
      cube([42, 42, 11], center=true);
      cylinder(r=2.5+1.4+0.25, h=1, center=false, $fn=30);
    } 
    
    //cover holes
    for(rz = [0:90:270]) rotate([0, 0, 45+rz]) hull(){
      translate([37, 0, 0]) cylinder(d=3.5, h=15, center=false, $fn=30);
    }    
  }    
}

module smaHolder(){
  
  difference(){
    hull(){
        translate([0, 0, 0]) cylinder(d=16, h=8, $fn=30, center=false);
        translate([12, 0, 0]) cylinder(d=16, h=8, $fn=30, center=false);
    } 
    
    hull(){
        translate([0, 0, 0]) cylinder(d=12.5, h=5, $fn=30, center=false);
        translate([12, 0, 0]) cylinder(d=12.5, h=5, $fn=30, center=false);
    } 
    
    translate([12, 0, 0]) cylinder(d=3.5, h=8, $fn=15, center=false);
  }
  
  difference(){
    intersection(){    
      translate([0, 0, 8]) hull(){
          translate([0, 0, 0]) cylinder(d=16, h=16, $fn=30, center=false);
          translate([12, 0, 0]) cylinder(d=16, h=16, $fn=30, center=false);
      } 
      translate([-3.5, 0, 16/2+8]) cube([13, 16, 16], center=true);
    }
    translate([-10, 0, 18]) rotate([90, 0, 90]){ 
      cylinder(d=7, h=20, $fn=20, center=false);
      cylinder(d=10.5, h=10, $fn=6, center=false);
    }
    
  }

}

module boxSeal(){
   intersection(){
     pcbCover();
     translate([0, 0, 1/2]) cube([60, 60, 1], center=true);
   }
}

module cableSeal(){
  translate([0, 0, 2])
  minkowski(){ 
    cube([11, 5, 4], center=true);
    cylinder(r=0.9, h=1, center=false, $fn=15);
  }
  translate([0, 0, 0.25])
  minkowski(){ 
    cube([11, 5, 0.5], center=true);
    cylinder(r=2, h=0.5, center=false, $fn=15);
  }
}

module cableSealSensor(){
  difference(){
    cableSeal();
    hull(){
      translate([-1.3/2, 0, 0]) cylinder(d=1.3, h=5, center=false, $fn=15);
      translate([1.3/2, 0, 0]) cylinder(d=1.3, h=5, center=false, $fn=15);
    }
    translate([-13/2, 0, 5/2]) cube([13, 0.2, 5], center=true);
  }
}

module cableSealPowerAntenna(){
  difference(){
    cableSeal();
    translate([-2, 0, 0])
    hull(){
      translate([-1.3/2, 0, 0]) cylinder(d=1.3, h=5, center=false, $fn=15);
      translate([1.3/2, 0, 0]) cylinder(d=1.3, h=5, center=false, $fn=15);
    }
    translate([2, 0, 0]) cylinder(d=1.8, h=5, center=false, $fn=15);
    translate([-13/2, 0, 5/2]) cube([13+5, 0.2, 5], center=true);
  }
}

module funnelSeal(){
  difference(){
    cylinder(d=2*(funnelRadius-9+4), h=1, center=false, $fn=fn);  
    cylinder(d=2*(funnelRadius-9), h=1, center=false, $fn=fn);  
  }
}


module magneticFoot(){
  for(rz = [0:90:270]) rotate([0, 0, 45+rz]) translate([funnelRadius-18, 0, -55]) difference(){
    cylinder(d=24, h=40, center=false, $fn=60);
    translate([0, 0, 22]) cylinder(d=3.3, h=18, center=false, $fn=60);
    translate([0, 0, 5.5]) cylinder(d=3.3, h=10, center=false, $fn=60);
    cylinder(d=20.5, h=5.5, center=false, $fn=60);
  }
}

//translate([0, -30, 30]) rotate([90, 90, 0]) pcb();
//funnel();
base();
//magneticFoot();
//bucketHolders();
//translate([0, 0, 15]) bucket();

//translate([0, -18, 42]) rotate([90, 90, 0]) pcbHolder();
//translate([0, -45, 42]) rotate([90, 90, 0]) pcbCover();



//translate([0, 22, 5]) sensor();
//rotate([0, 0, 90]) translate([0, 0, -20]) extensionHarm(35);
//solarCellHolder();
//translate([-58, 20, 6.5]) 
//rotate([0, 0, 0]) smaHolder();


//translate([50, 0, 0]) pcbHolder();
//translate([-50, 0, 0]) pcbCover();
/*
boxSeal();
translate([-10, 0, 0]) cableSealPowerAntenna();
translate([10, 0, 0]) cableSealSensor();
funnelSeal();
*/




